/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.2 Baustellensimulation
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.simbaustelle.verwaltung;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ibv.simbaustelle.simbaustelle.BaustellenSimulation;
import de.bsvrz.ibv.simbaustelle.simbaustelle.BssKonstanten;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * eine Hilfsobjekt zur Verarbeitung der Kommandos zur Ausführung einer
 * Simulation auf Nutzeranforerung.<br>
 * Die Klasse wertet die über die Attributgruppe
 * "atg.baustellenSimulationStarten" empfangenen Daten aus und löst eine
 * sofortige Berechnung der entsprechenden Baustelle aus.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class BaustellenSimulationsObserver implements ClientReceiverInterface {

	/**
	 * Logger für Debugausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * die Attributgruppe zum Empfang des Kommandos zum Start der Simulation.
	 */
	private static AttributeGroup startAtg;

	/**
	 * die Attributgruppe zum Empfang des Kommandos zum Start der Simulation mit
	 * vorgegebenem MQ.
	 */
	private static AttributeGroup startMitMqAtg;

	/**
	 * der erwartetet Aspekt.
	 */
	private static Aspect startAsp;

	/**
	 * die Baustelle, die gesteuert werden soll.
	 */
	private final BssBaustelle baustelle;

	private ClientDavInterface connection;

	/**
	 * erzeugt eine Instanz des Steuerungsobjekts.
	 *
	 * @param baustelle
	 *            die zu steuernde Baustelle
	 */
	public BaustellenSimulationsObserver(final BssBaustelle baustelle) {
		connection = BaustellenSimulation.getInstance().getVerbindung();
		this.baustelle = baustelle;

		if (BaustellenSimulationsObserver.startAtg == null) {
			final DataModel model = connection.getDataModel();
			BaustellenSimulationsObserver.startAtg = model
					.getAttributeGroup(BssKonstanten.PID_ATG_BAUSTELLEN_SIMULATION_STARTEN);
			BaustellenSimulationsObserver.startMitMqAtg = model
					.getAttributeGroup(BssKonstanten.PID_ATG_BAUSTELLEN_SIMULATION_STARTEN_MIT_MQ);
			BaustellenSimulationsObserver.startAsp = model.getAspect(BssKonstanten.PID_ASP_SENDEN);
		}

		connection.subscribeReceiver(this, baustelle.getSystemObjekt(),
				new DataDescription(BaustellenSimulationsObserver.startAtg, BaustellenSimulationsObserver.startAsp),
				ReceiveOptions.normal(), ReceiverRole.drain());

		if (BaustellenSimulationsObserver.startMitMqAtg != null) {
			connection
					.subscribeReceiver(this, baustelle.getSystemObjekt(),
							new DataDescription(BaustellenSimulationsObserver.startMitMqAtg,
									BaustellenSimulationsObserver.startAsp),
							ReceiveOptions.normal(), ReceiverRole.drain());
		}
	}

	/**
	 * Die Funktion empfängt den Datensatz zur Ausführung einer Simulation und führt
	 * die Simulation der Baustelle aus.
	 *
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public final void update(final ResultData[] results) {
		for (final ResultData result : results) {
			LOGGER.finer("Befehl zum manuellen Simulationsstart empfangen", result);

			final Data data = result.getData();
			if (data != null) {
				if (result.getDataDescription().getAttributeGroup().equals(BaustellenSimulationsObserver.startAtg)) {
					final String name = data.getTextValue(BssKonstanten.ATT_NAME_SIMSTART_NAME).getText();
					final String bemerkung = data.getTextValue(BssKonstanten.ATT_NAME_SIMSTART_BEMERKUNG).getText();
					BaustellenSimulation.getInstance()
							.addAuftrag(new ManuellerAuftrag(baustelle, name, bemerkung, null));
				} else if (result.getDataDescription().getAttributeGroup()
						.equals(BaustellenSimulationsObserver.startMitMqAtg)) {
					final String name = data.getTextValue(BssKonstanten.ATT_NAME_SIMSTART_NAME).getText();
					final String bemerkung = data.getTextValue(BssKonstanten.ATT_NAME_SIMSTART_BEMERKUNG).getText();
					final SystemObject mqObj = data.getReferenceValue(BssKonstanten.ATT_NAME_SIMSTART_BERECHNUNGS_MQ)
							.getSystemObject();

					MessQuerschnittAllgemein berechnungsMq = null;
					if (mqObj != null) {
						berechnungsMq = (MessQuerschnittAllgemein) ObjektFactory.getInstanz().getModellobjekt(mqObj);
					}
					BaustellenSimulation.getInstance()
							.addAuftrag(new ManuellerAuftrag(baustelle, name, bemerkung, berechnungsMq));
				}
			}
		}
	}

	/** entfernt die Datenanmeldungen des Observers. */
	public void disconnect() {
		connection.unsubscribeReceiver(this, baustelle.getSystemObjekt(),
				new DataDescription(BaustellenSimulationsObserver.startAtg, BaustellenSimulationsObserver.startAsp));
		if (BaustellenSimulationsObserver.startMitMqAtg != null) {
			connection.unsubscribeReceiver(this, baustelle.getSystemObjekt(), new DataDescription(
					BaustellenSimulationsObserver.startMitMqAtg, BaustellenSimulationsObserver.startAsp));
		}
	}
}
